
import UIKit
import Photos
import PhotosUI

class PhotoEditingViewController: HipsterizeViewController, PHContentEditingController {
  var input: PHContentEditingInput?

  override func viewDidLoad() {
    super.viewDidLoad()
    // Do any additional setup after loading the view.
  }

  override func didReceiveMemoryWarning() {
    super.didReceiveMemoryWarning()
    // Dispose of any resources that can be recreated.
  }

  override func viewDidAppear(animated: Bool) {
    super.viewDidAppear(animated)

    // auto-hipsterize the photo
    let hipsterize = Hipsterize(image: imageView.image!)
    hipsterElements = hipsterize.hipsterElements()

    // add hipster elements to the view
    for element in hipsterElements {
      element.createViewForImageFrame(imageFrame)
      if let elementView = element.view {
        element.active = true
        view.addSubview(elementView)
      }
    }

    addPaletteElements()
  }

  // MARK: - PHContentEditingController

  func canHandleAdjustmentData(adjustmentData: PHAdjustmentData?) -> Bool {
    return false
  }

  func startContentEditingWithInput(contentEditingInput: PHContentEditingInput?, placeholderImage: UIImage) {
    input = contentEditingInput
    imageView.image = input!.displaySizeImage
  }

  func finishContentEditingWithCompletionHandler(completionHandler: ((PHContentEditingOutput!) -> Void)!) {
    removePaletteElements()
    setUIHidden(true)
    tapRecognizer.enabled = false
    panRecognizer.enabled = false
    pinchRecognizer.enabled = false
    rotationRecognizer.enabled = false

    // Render and provide output on a background queue.
    dispatch_async(dispatch_get_global_queue(CLong(DISPATCH_QUEUE_PRIORITY_DEFAULT), 0)) {
      let output = PHContentEditingOutput(contentEditingInput: self.input)

      let inputImage = UIImage(contentsOfFile: self.input!.fullSizeImageURL.path!)
      let hipsterize = Hipsterize(image: inputImage!)
      let outputImage = hipsterize.imageWithRenderedElements(self.hipsterElements)

      let jpegData = UIImageJPEGRepresentation(outputImage, 0.8)
      jpegData.writeToURL(output.renderedContentURL, atomically: true)

      output.adjustmentData = PHAdjustmentData(
        formatIdentifier: "com.razeware.hipsterize",
        formatVersion: "2.0",
        data: NSKeyedArchiver.archivedDataWithRootObject(self.hipsterElements))

      completionHandler?(output)
    }
  }

    var shouldShowCancelConfirmation: Bool {
        // Determines whether a confirmation to discard changes should be shown to the user on cancel.
        // (Typically, this should be "true" if there are any unsaved changes.)
        return false
    }

    func cancelContentEditing() {
        // Clean up temporary files, etc.
        // May be called after finishContentEditingWithCompletionHandler: while you prepare output.
    }

}
